<?php

namespace Drupal\Tests\smart_read_more_link\Unit;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Entity\EntityBase;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Drupal\smart_read_more_link\Plugin\Field\FieldFormatter\SmartReadMoreLinkFormatter;
use Drupal\Tests\UnitTestCase;

/**
 * Unit tests of smart_read_more_link module.
 *
 * @group smart_read_more_link
 */
class SmartReadMoreLinkUnitTest extends UnitTestCase {

  /**
   * Test that the formatter delegates its settings as expected.
   */
  public function testDelegationOfSettings() {
    $field_definition = $this->createMock(FieldDefinitionInterface::class);
    $base_formatter = $this->createMock(FormatterBase::class);

    $random_string1 = $this->getRandomGenerator()->string(20);
    $random_string2 = $this->getRandomGenerator()->string(20);
    $base_formatter->method('settingsForm')->willReturn([
      '#type' => 'markup',
      '#markup' => $random_string1,
    ]);
    $base_formatter->method('settingsSummary')->willReturn($random_string2);
    $default_formatter = $this->createMock(FormatterBase::class);
    $formatter_manager = $this->createMock(PluginManagerInterface::class);
    $formatter_manager->method('createInstance')
      ->willReturnCallback(static function ($plugin_id) use ($default_formatter, $base_formatter) {
        if ($plugin_id === 'text_summary_or_trimmed') {
          return $base_formatter;
        }
        else {
          return $default_formatter;
        }
      });
    $renderer = $this->createMock(RendererInterface::class);

    $formatter = new SmartReadMoreLinkFormatter(
      'smart_read_more_link',
      [],
      $field_definition,
      [],
      'Field label',
      'view_mode',
      [],
      $formatter_manager,
      $renderer
    );
    $form_state = $this->createMock(FormState::class);
    $settings_form = $formatter->settingsForm([], $form_state);
    $this->assertEquals($settings_form['#markup'], $random_string1, 'delegates form to base formatter');
    $settings_summary = $formatter->settingsSummary();
    $this->assertEquals($settings_summary, $random_string2, 'delegates summary to base formatter');
  }

  /**
   * Test no link is generated when teaser and full views match.
   */
  public function testWhenTeaserMatchesFull() {
    $field_definition = $this->createMock(FieldDefinitionInterface::class);
    $base_formatter = $this->createMock(FormatterBase::class);

    $random_string1 = $this->getRandomGenerator()->string(20);
    $default_formatter = $this->createMock(FormatterBase::class);
    $formatter_manager = $this->createMock(PluginManagerInterface::class);
    $formatter_manager->method('createInstance')
      ->willReturnCallback(static function ($plugin_id) use ($default_formatter, $base_formatter) {
        if ($plugin_id === 'text_summary_or_trimmed') {
          return $base_formatter;
        }
        else {
          return $default_formatter;
        }
      });
    $renderer = $this->createMock(RendererInterface::class);
    // Rendered view is constant.
    $renderer->method('render')
      ->willReturn($random_string1);
    $formatter = new SmartReadMoreLinkFormatter(
      'smart_read_more_link',
      [],
      $field_definition,
      [],
      'Field label',
      'view_mode',
      [],
      $formatter_manager,
      $renderer
    );

    $items = $this->createMock(FieldItemListInterface::class);
    $result = $formatter->viewElements($items, 'und');
    // @todo be more thorough about this
    $this->assertEmpty($result[0]['links'], 'no link in output');
  }

  /**
   * Test no link is generated when teaser and full views match.
   */
  public function testWhenTeaserDoesNotMatchFull() {
    $field_definition = $this->createMock(FieldDefinitionInterface::class);
    $base_formatter = $this->createMock(FormatterBase::class);
    $base_formatter->method('viewElements')->willReturn([['#markup' => 'anything']]);
    $random_string1 = $this->getRandomGenerator()->string(200);
    $random_string2 = $this->getRandomGenerator()->string(100);
    $default_formatter = $this->createMock(FormatterBase::class);
    $default_formatter->method('viewElements')->willReturn([['#markup' => 'otherthing']]);
    $formatter_manager = $this->createMock(PluginManagerInterface::class);
    $formatter_manager->method('createInstance')
      ->willReturnCallback(function ($plugin_id) use ($default_formatter, $base_formatter) {
        if ($plugin_id === 'text_summary_or_trimmed') {
          return $base_formatter;
        }
        else {
          return $default_formatter;
        }
      });
    $renderer = $this->createMock(RendererInterface::class);
    // Rendered view is constant.
    $renderer->method('render')
      ->willReturnOnConsecutiveCalls($random_string1, $random_string2);
    $formatter = new SmartReadMoreLinkFormatter(
      'smart_read_more_link',
      [],
      $field_definition,
      [],
      'Field label',
      'view_mode',
      [],
      $formatter_manager,
      $renderer
    );
    $entity = $this->createMock(EntityBase::class);

    $entity->method('label')->willReturn('Entity label');
    $url = $this->createMock(Url::class);
    $entity->method('urlInfo')->willReturn($url);
    $entity->method('language')->willReturn('und');
    $items = $this->createMock(FieldItemListInterface::class);
    $items->method('getEntity')->willReturn($entity);
    $formatter->setStringTranslation($this->getStringTranslationStub());
    $result = $formatter->viewElements($items, 'und');

    // @todo be more thorough about this
    $this->assertNotEmpty($result[0]['links'], 'link in output');
  }

}
